local g = vim.g
local opt = vim.opt

g.mapleader = ' '
g.tex_flavor = 'latex'
g.c_syntax_for_h = true

--opt.wildoptions = true -- TODO: see how it works
opt.termguicolors = true
opt.guicursor = 'n-v-c-sm:hor20,i-ci-ve:ver20,r-cr-o:Block'
opt.mouse = 'nv'
opt.path = '.,,**h'
-- opt.pumheight = 10 -- TODO: probably to change
opt.fileencoding = 'utf-8'
opt.splitbelow = true
opt.splitright = true
opt.smartindent = false -- TODO: maybe?
opt.showmode = false
opt.undofile = true
opt.updatetime = 300
opt.backup = false
opt.clipboard:prepend { 'unnamedplus' }
opt.smartcase = true
opt.cursorline = true
opt.sessionoptions = 'folds'
opt.foldmethod = 'marker'
opt.foldlevel = 0
opt.number = true
opt.hidden = true
opt.shortmess:append('I', 'W', 's')
-- opt.scrolloff = 15 -- TODO: maybe ?
opt.iskeyword:prepend { '-' }
opt.inccommand = 'nosplit'
opt.completeopt:append 'menuone'
opt.timeoutlen = 300
opt['foldenable'] = false
opt.fillchars = 'eob: '
opt.cmdheight = 1
opt.grepprg = 'rg --vimgrep --no-heading --smart-case --hidden'
opt.grepformat = '%f:%l:%c:%m'
-- opt.autochdir = true

vim.api.nvim_create_user_command('Cd', 'lcd %:p:h', { nargs = 0 })

local ignore_buftype = { 'quickfix', 'nofile', 'help' }
local ignore_filetype = { 'gitcommit', 'gitrebase', 'svn', 'hgcommit' }

local function run()
	if vim.tbl_contains(ignore_buftype, vim.bo.buftype) then
		return
	end

	if vim.tbl_contains(ignore_filetype, vim.bo.filetype) then
		-- reset cursor to first line
		vim.cmd [[normal! gg]]
		return
	end

	-- If a line has already been specified on the command line, we are done
	--   nvim file +num
	if vim.fn.line '.' > 1 then
		vim.cmd [[normal! zz]]
		return
	end

	local last_line = vim.fn.line [['"]]
	local buff_last_line = vim.fn.line '$'

	-- If the last line is set and the less than the last line in the buffer
	if last_line > 0 and last_line <= buff_last_line then
		local win_last_line = vim.fn.line 'w$'
		local win_first_line = vim.fn.line 'w0'
		-- Check if the last line of the buffer is the same as the win
		if win_last_line == buff_last_line then
			-- Set line to last line edited
			vim.cmd [[normal! g`"]]
		-- Try to center
		elseif buff_last_line - last_line > ((win_last_line - win_first_line) / 2) - 1 then
			vim.cmd [[normal! g`"zz]]
		else
			vim.cmd [[normal! G'"<c-e>]]
		end
	end
end

vim.api.nvim_create_autocmd({ 'BufWinEnter', 'FileType' }, {
	group = vim.api.nvim_create_augroup('nvim-lastplace', {}),
	callback = run,
})
